///
/// @file  GUI.cpp
/// @param Project: TestGraph
/// @author Erik Zorzin
/// @date 22NOV2015
///

#include "GUI.hpp"

enum SlidersIDs                                                                                                                 // Slider ID enum.
{
    A_sld_ID                                                                                                                    // "Amplitude" [m] slider.
};

GUI::GUI(const TGWindow* p, UInt_t w, UInt_t h)
{
    fMain = new TGMainFrame(p, w, h);                                                                                           // Creating main frame...
    fEcanvas = new TRootEmbeddedCanvas("Ecanvas", fMain, w, h);                                                                 // Creating canvas...
    fMain->AddFrame(fEcanvas, new TGLayoutHints(kLHintsExpandX|kLHintsExpandY, 10, 10, 10, 1));                                 // Adding canvas to main frame...
    fMain->SetWindowName("TestGraph");                                                                                          // Setting main frame name...
    hframe = new TGHorizontalFrame(fMain, 600, 40);                                                                             // Creating hframe...
    A_sld = new TGHSlider(hframe, 150, kSlider1|kScaleDownRight, A_sld_ID);                                                     // Creating "A_sld" slider...
    A_sld->Connect("PositionChanged(Int_t)", "GUI", this, "DoSlider(Int_t)");                                                   // Connecting "A_sld" silder with "Do_A_sld" method...
    A_sld->SetRange(0, 100);                                                                                                    // Setting the range of the slider...
    hframe->AddFrame(A_sld, new TGLayoutHints(kLHintsCenterX, 5, 5, 3, 4));                                                     // Adding slider to hframe...
    fMain->AddFrame(hframe, new TGLayoutHints(kLHintsCenterX, 2, 2, 2, 2));                                                     // Adding hframe to main frame...
    fMain->Connect("CloseWindow()", "GUI", this, "CloseApplication()");                                                         // Closing the window will terminate the application...
    pad = new TPad("p1", "p1", 0.05, 0.05, 0.95, 0.95);                                                                         // Creating a pad...
    watch = new TStopwatch();
    
    
    /*-------------------------------------------------------------------------------------------------------------------------*/
    /*----------------------------------------------------- CREATING A GRAPH --------------------------------------------------*/
    /*-------------------------------------------------------------------------------------------------------------------------*/
    graph = new TGraph2D();                                                                                                     // Creating new scatter plot graph...
    graph->SetLineColor(2);                                                                                                     // Setting line color...
    graph->SetLineWidth(1);                                                                                                     // Setting line width...
    graph->SetTitle("sin(x)");                                                                                                  // Setting graph title...
    graph->GetXaxis()->SetTitle("x");                                                                                           // Setting x-axis title...
    graph->GetYaxis()->SetTitle("sin(x)");                                                                                      // Setting y-axis title...
    graph->SetMarkerColor(4);                                                                                                   // Setting marker color...
    graph->SetMarkerStyle(20);                                                                                                  // Setting marker style...
    graph->GetXaxis()->SetLimits(0, 1);                                                                                         // Setting "X" limits...
    graph->SetMinimum(-1.5);                                                                                                    // Setting "Y" minimum...
    graph->SetMaximum(1.5);                                                                                                     // Setting "Y" maximum...
    
    /*-------------------------------------------------------------------------------------------------------------------------*/
    /*----------------------------------------------------- DRAWING OBJECTS ---------------------------------------------------*/
    /*-------------------------------------------------------------------------------------------------------------------------*/
    fCanvas = fEcanvas->GetCanvas();                                                                                            // Getting the canvas...
    fCanvas->Clear();                                                                                                           // Clearing the canvas...
    fCanvas->cd();                                                                                                              // Going into canvas...
    pad->Draw();                                                                                                                // Drawing pad...
    pad->cd();                                                                                                                  // Going into pad...
    
    /*-------------------------------------------------------------------------------------------------------------------------*/
    /*----------------------------------------------------- UPDATING OBJECTS --------------------------------------------------*/
    /*-------------------------------------------------------------------------------------------------------------------------*/
    fMain->MapSubwindows();                                                                                                     // Mapping all subwindows of the main frame...
    fMain->Resize(fMain->GetDefaultSize());                                                                                     // Initializing the layout algorithm...
    fMain->MapWindow();                                                                                                         // Mapping main frame...
}

GUI::~GUI()
{
    fMain->Cleanup();                                                                                                           // Cleaning up used widgets: frames, buttons, layout hints...
    delete fMain;                                                                                                               // Deleting fMain...
    gApplication->Terminate(0);                                                                                                 // Sending "Terminate" signal to the application...
}

void GUI::DoSlider(Int_t pos)
{
    A = pos/100.0;                                                                                                              // Calculating "Amplitude" [m]...
}

void GUI::Plot()
{
    watch->Start();
    
    phi_x += PHI;
    phi_y += PHI;
    
    if (phi_x > (2*PI - PHI))
    {
        phi_x = 0;
    }
    
    if (phi_y > (2*PI - PHI))
    {
        phi_y = 0;
    }
    
    graph->Clear();                                                                                                             // Clearing the graph...
    
    for (i = 0; i < N; i++)
    {
        for (j = 0; j < M; j++)
        {
            k = j + i*N;                                                                                                        // Calulating index [#]...
            x = i;                                                                                                              // Calculating "x" values...
            y = j;                                                                                                              // Calculating "y" values...
            z = A*sin(6.28*i/M + phi_x)*cos(6.28*j/N + phi_y);                                                                  // Calculating "z" values...
            graph->SetPoint(k, x, y, z);                                                                                        // Updating graph points...
        }
    }
    printf("Elapsed time (computation) [s] = %lf\n", watch->CpuTime());
    
    watch->Start();
    //fCanvas->SetEditable(kTRUE);                                                                                              // Enabling canvas editing...
    graph->Draw("surf1");                                                                                                       // Drawing the graph...
    fCanvas->Modified();                                                                                                        // Setting "Modified" bit...
    fCanvas->Update();                                                                                                          // Updating the canvas...
    //fCanvas->SetEditable(kFALSE);                                                                                             // Disabling canvas editing...
    printf("Elasped time (visualization) [s] = %lf\n", watch->CpuTime());
    
}

void GUI::CloseApplication()
{
    gApplication->Terminate(0);                                                                                                 // Sending "Terminate" signal to the application...
}



